///////////////////////////////////////////////////////////////////////////////
// BOSSA
//
// Copyright (c) 2011-2018, ShumaTech
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the <organization> nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
///////////////////////////////////////////////////////////////////////////////
#ifndef _BOSSAWINDOW_H
#define _BOSSAWINDOW_H

#include <wx/wx.h>

#include "BossaForm.h"
#include "BossaThread.h"
#include "BossaProgress.h"

class BossaWindow : public MainFrame
{
public:
    BossaWindow();
    virtual ~BossaWindow();

private:
    BossaThread* _thread;
    BossaProgress* _progress;

    void RefreshSerial();
    void CreateDevice();
    void Connected();
    void Disconnected();

    bool GetOffset(uint32_t& offset);

    void Error(const wxString& message);
    void Warning(const wxString& message);
    void Info(const wxString& message);
    bool Question(const wxString& message);

    void OnAbout(wxCommandEvent& event);
    void OnRefresh(wxCommandEvent& event);
    void OnSerial(wxCommandEvent& event);

    void OnWrite(wxCommandEvent& event);
    void OnVerify(wxCommandEvent& event);
    void OnRead(wxCommandEvent& event);
    void OnInfo(wxCommandEvent& event);
    void OnExit(wxCommandEvent& event);

    void OnThreadProgress(wxCommandEvent& event);
    void OnThreadSuccess(wxCommandEvent& event);
    void OnThreadWarning(wxCommandEvent& event);
    void OnThreadError(wxCommandEvent& event);

    void OnProgressCancel(wxCommandEvent& event);
};

#endif // _BOSSAWINDOW_H
